// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#if !defined(_CIW_GAME_SPRITE_H_)
#define _CIW_GAME_SPRITE_H_

#include "s3e.h"
#include "Iw2D.h"

#include "IwGameAnim.h"
#include "IwGameImage.h"
#include "IwGameFont.h"
#include "IwGameBrush.h"

class CIwGameSpriteManager;

//
//
//
// CIwGameSprite - A sprite is the visual representation of an on screen game object
//
// Can be thought of as an interface rather than a concrete class as other sprite types are created from this
//
//
class CIwGameSprite
{
	/// Properties
protected:
	CIwGameSpriteManager*	Parent;					// Parent sprite manager
	int						Width, Height;			// Destination width and height (used to represent the visible extents of the sprite on screen)
	iwfixed					Depth;					// Depth
	CIwMat2D				Transform;				// Transform
	CIwVec2					Position;				// Position of the sprite
	CIwVec2					Origin;					// Origin of sprite (0, 0 is sprites centre)
	iwangle					Angle;					// Rotation of sprite (IW_ANGLE_2PI = 360 degrees)
	iwfixed					ScaleX;					// X axis scale of sprite (IW_GEOM_ONE = 1.0)
	iwfixed					ScaleY;					// X axis scale of sprite (IW_GEOM_ONE = 1.0)
	CIwColour				Colour;					// Colour of sprite
	bool					Visible;				// Sprites visible state
	bool					Pooled;					// Tells system if we belong to a sprite pool or not
	bool					InUse;					// Used in a memory pooling system to mark this sprite as in use
	int						Layer;					// Depth layer
	CIwGameSprite*			LinkedTo;				// When a sprite is linked to another it takes on that sprites transform etc..
public:
	void		setParent(CIwGameSpriteManager* parent) { Parent = parent; }
	CIwGameSpriteManager* getParent()		{ return Parent; }
	void		setDestSize(int width, int height)
	{
		Width = width;
		Height = height;
	}
	void		setDepth(iwfixed depth)		{ Depth = depth; }
	int			getDestWidth() const		{ return Width; }
	int			getDestHeight() const		{ return Height; }
	iwfixed		getDepth() const			{ return Depth; }
	void		setPosAngScale(int x, int y, iwangle angle, iwfixed scale)
	{
		Position.x = x;
		Position.y = y;
		Angle = angle;
		ScaleX = ScaleY = scale;
		TransformDirty = true;
	}
	void		setPosAngScale(int x, int y, iwangle angle, iwfixed scale_x, iwfixed scale_y)
	{
		Position.x = x;
		Position.y = y;
		Angle = angle;
		ScaleX = scale_x;
		ScaleY = scale_y;
		TransformDirty = true;
	}
	void		setPosition(int x, int y)
	{
		Position.x = x;
		Position.y = y;
		TransformDirty = true;
	}
	CIwVec2		getPosition() const			{ return Position; }
	void		setOrigin(int x, int y)
	{
		Origin.x = x;
		Origin.y = y;
		TransformDirty = true;
	}
	CIwVec2		getOrigin() const			{ return Origin; }
	void		setAngle(iwangle angle)
	{
		Angle = angle;
		TransformDirty = true;
	}
	void		setAngle(float angle)
	{
		Angle = (iwangle)(angle * IW_ANGLE_2PI);
		TransformDirty = true;
	}
	iwangle		getAngle() const			{ return Angle; }		
	void		setScale(iwfixed scale)
	{
		ScaleX = scale;
		ScaleY = scale;
		TransformDirty = true;
	}
	void		setScale(float scale)
	{
		ScaleX = (iwfixed)(scale * IW_GEOM_ONE);
		ScaleY = ScaleX;
		TransformDirty = true;
	}
	void		setScale(iwfixed scale_x, iwfixed scale_y)
	{
		ScaleX = scale_x;
		ScaleY = scale_y;
		TransformDirty = true;
	}
	void		setScale(float scale_x, float scale_y)
	{
		ScaleX = (iwfixed)(scale_x * IW_GEOM_ONE);
		ScaleY = (iwfixed)(scale_y * IW_GEOM_ONE);
		TransformDirty = true;
	}
	iwfixed		getScale() const			{ return ScaleX; }
	iwfixed		getScaleX() const			{ return ScaleX; }
	iwfixed		getScaleY() const			{ return ScaleY; }
	virtual void setColour(CIwColour& colour) { Colour = colour; }
	CIwColour	getColour() const			{ return Colour; }
	void		setVisible(bool show)		{ Visible = show; }
	bool		isVisible() const			{ return Visible; }
	void		forceTransformDirty()		{ TransformDirty = true; }
	void		setPooled(bool pooled)		{ Pooled = pooled; }
	bool		isPooled() const			{ return Pooled; }
	void		setInUse(bool in_use)		{ InUse = in_use; }
	bool		isInUse() const				{ return InUse; }
	void		setLayer(int layer)			{ Layer = layer; }
	int			getLayer() const			{ return Layer; }
	CIwMat2D&	getTransform()				{ return Transform; }
	void		setLinkedTo(CIwGameSprite* sprite) { LinkedTo = sprite; }
	CIwGameSprite* getLinkedTo()			{ return LinkedTo; }
	iwfixed		getAccumDepth() const		{ return AccumDepth; }
	/// Properties End
protected:
	CIwMat2D	FinalTransform;				// Final transform
	bool		TransformDirty;				// Dirty when transform changed
	CIwVec2		TransformedV[4];			// Transforrmed vertices (used for hit detection)
	iwfixed		AccumDepth;					// Accumulated depth

	virtual void	RebuildTransform();		// Rebuilds the display transform
	virtual void	TransformVertices();	// Builds a list of transformed vertices

public:
	CIwGameSprite() : Pooled(false)			{ Init(); }
	virtual ~CIwGameSprite() {}

	virtual void	Init();					// Called to initialise the sprite, used after construction or to reset the sprite in a pooled sprite system
	virtual bool	Update();				// Updates the sprite
	virtual void	Draw() = 0;				// Pure virtual, need to implement in derived classes
	virtual bool	HitTest(int x, int y);	// Check to see if point is within area covered by transformed sprite
	virtual CIwVec2 TransformPoint(float x, float y);	// Transform point by sprites local angle / scale transform

	virtual void	RebuildTransformNow();	// Rebuilds the display transform immediately
};

//
//
//
// CIwGameDummySprite - A DummySprite is a visual that has no visual component and is usually only used for its attributes
//
//
//
class CIwGameDummySprite : public CIwGameSprite
{
	// Properties
protected:
public:
	// Properties End
public:
	CIwGameDummySprite() : CIwGameSprite()	{}
	virtual ~CIwGameDummySprite() {}
	
	void	Draw();
};

//
//
//
// CIwGameBitmapSprite - A BitmapSprite is a bitmapped visual representation of an on screen game object
//
//
//
class CIwGameBitmapSprite : public CIwGameSprite
{
	// Properties
protected:
	CIwGameImage*		Image;					// Bitmapped image that represents this sprite
	int					SrcX, SrcY;				// Top left position in source texture
	int					SrcWidth, SrcHeight;	// Width and height of sprite in source texture
	CIw2DImageTransform	ImageTransform;			// Marmalade image transform
	bool				UVsDirty;				// Dirty when source texture has changed
public:
	void		setImage(CIwGameImage* image)
	{
		Image = image;
		UVsDirty = true;
	}
	CIwGameImage*	getImage()										{ return Image; }
	void		setColour(CIwColour& colour)
	{
		Colour = colour;
		uint32 c = colour.Get();
		Colours[0] = c;
		Colours[1] = c;
		Colours[2] = c;
		Colours[3] = c;
	}
	void		setColour(int index, CIwColour& colour)
	{
		Colours[index] = colour.Get();
	}
	void		setSrcDest(int x, int y, int width, int height)
	{
		setDestSize(width, height);
		SrcX = x;
		SrcY = y; 
		SrcWidth = width;
		SrcHeight = height;
		UVsDirty = true;
	}
	void		setSrcRect(int x, int y, int width, int height)
	{
		SrcX = x;
		SrcY = y; 
		SrcWidth = width;
		SrcHeight = height;
		UVsDirty = true;
	}
	void		setSrcRect(CIwRect* src)
	{
		SrcX = src->x;
		SrcY = src->y; 
		SrcWidth = src->w;
		SrcHeight = src->h;
		UVsDirty = true;
	}
	CIwRect		getSrcRect() const									{ return CIwRect(SrcX, SrcY, SrcWidth, SrcHeight); }
	int			getSrcWidth() const									{ return SrcWidth; }
	int			getSrcHeight() const								{ return SrcHeight; }
	void		setFromBrush(CIwGameBrushImage* brush);
	void		setImageTransform(CIw2DImageTransform transform)	{ ImageTransform = transform; }
	CIw2DImageTransform getImageTransform() const					{ return ImageTransform; }
	// Properties End

protected:
	CIwSVec2			UVList[4];					// Texture UV coordinate list
	CIwColour			Colours[4];					// Vertices colours
	void				RebuildUVList();

public:

	CIwGameBitmapSprite() : CIwGameSprite(), Image(NULL), SrcX(0), SrcY(0), SrcWidth(0), SrcHeight(0), ImageTransform(IW_2D_IMAGE_TRANSFORM_NONE)	{}
	virtual ~CIwGameBitmapSprite() {}
	
	void	Draw();
};

//
//
//
// CIwGameTextSprite - A CIwGameTextSprite is a text based visual representation of an on screen game object
//
//
//
class CIwGameTextSprite : public CIwGameSprite
{
	// Properties
protected:
	CIwGameString		Text;					// Text to display
	CIwRect				Rect;					// Area where to draw the text
	CIwGameFont*		Font;					// Font
	bool				TextDirty;				// Dirty when text has changed
	IwGxFontFlags		Flags;					// Font flags
	IwGxFontAlignHor	AlignH;					// Horiznotal alignment
	IwGxFontAlignVer	AlignV;					// Vertical alignment
public:
	void		setText(const char* text)
	{
		Text = text;
		TextDirty = true;
	}
	CIwGameString&	getText()										{ return Text; }
	void			setRect(CIwRect& rect)							{ Rect = rect; TransformDirty = true; }
	CIwRect&		getRect()										{ return Rect; }
	void			setFont(CIwGameFont* font)
	{
		Font = font;
		TextDirty = true;
	}
	CIwGameFont*	getFont()										{ return Font; }
	void			setFlags(IwGxFontFlags flags)					{ Flags = flags; }
	void			setAlignH(IwGxFontAlignHor align)				{ AlignH = align; }
	void			setAlignV(IwGxFontAlignVer align)				{ AlignV = align; }
	// Properties End

protected:
	CIwGxFontPreparedData PreparedText;
	void			RebuildText();
	void			RebuildTransform();
	void			UpdateBindings();
	void			TransformVertices();

public:

	CIwGameTextSprite() : CIwGameSprite(), Font(NULL), Flags(IW_GX_FONT_DEFAULT_F), AlignH(IW_GX_FONT_ALIGN_CENTRE), AlignV(IW_GX_FONT_ALIGN_MIDDLE), TextDirty(true)   {}
	virtual ~CIwGameTextSprite() {}
	
	void	Draw();
};

//
//
// CIwGameSpriteManager - A sprite manager 
//
// The sprite manager managers a collection of sprites, including drawing, tracking and clean up
// The sprite manager also carries its own visual transform that will be applied to all of its children, allowing the user to apply rotation, scaling ans translation to all child sprites
//
//
class CIwGameSpriteManager
{
public:
	// Provide public access to iteration of the sprite list
	typedef CIwList<CIwGameSprite*>::iterator	Iterator;
	Iterator		begin() { return Sprites.begin(); }
	Iterator		end()	{ return Sprites.end(); }

	// Properties
protected:
	CIwMat2D					Transform;			// Transform
	CIwList<CIwGameSprite*>		Sprites;			// Our list of sprites
	CIwList<CIwGameSprite*>*	Layers;				// Visible layers used in depth sorting
	bool						Batching;			// Enable sprite batching
	CIwSVec2					COP;				// Centre of projection (3 bit fixed)
	CIwSVec2					ScreenCentre;		// Screen centre (3 bit fixed)
public:
	void			addSprite(CIwGameSprite* sprite);
	void			removeSprite(CIwGameSprite* sprite, bool delete_sprites = true);
	void			setTransform(const CIwMat2D& transform)	{ Transform = transform; DirtyChildTransforms(); }
	const CIwMat2D&	getTransform() const					{ return Transform; }
	void			setBatching(bool batching)				{ Batching = batching; }
	bool			getBatching() const						{ return Batching; }
	void			setCOP(int x, int y)					{ COP.x = x; COP.y = y; }
	CIwSVec2		getCOP() const							{ return COP; }
	void			setScreenCentre(int x, int y)			{ ScreenCentre.x = x; ScreenCentre.y = y; }
	CIwSVec2		getScreenCentre() const					{ return ScreenCentre; }
	// Properties End

protected:
	void			DirtyChildTransforms();		// Dirties all child transforms to force them to update
	int				MaxLayers;					// Maximum layers
	void			ClearLayers();				// Clears all visible layers ready for next frame

public:
	CIwGameSpriteManager() : MaxLayers(0), Batching(true)
	{
		// Set up default rotation, scaling and translation
		Transform.SetIdentity();
		Transform.m[0][0] = IW_GEOM_ONE;
		Transform.m[1][1] = IW_GEOM_ONE;
	}
	~CIwGameSpriteManager() { Release(); }

	void			Init(int max_layers = 10);
	void			Draw();
	void			Release(bool delete_sprites = true);
};





#endif // _CIW_GAME_SPRITE_H_
