﻿// 
//
// IwGame - Cross Platform Multi-purpose Game Engine using the Marmalade SDK
//
// Developed by Matthew Hopwood of Pocketeers Limited - www.pocketeers.co.uk
//
// For updates, tutorials and more details check out my blog at www.drmop.com
//
// This code is provided free of charge and without any warranty whatsoever. The only restriction to its usage is that this header must remain intact and visible in all IwGame engine files.
// If you use this engine in your product, whilst it is not mandatory, a mention of the IwGame engine would be nice or a quick email to let us know where it is being used.
//
//

#include "IwGameActor.h"
#include "IwGameScene.h"
#include "IwGameCamera.h"
#include "IwGameXoml.h"
#include "IwGameActions.h"
#include "IwGame.h"
#include "IwGameBrush.h"


//
//
//
// CIwGameActor implementation
//
//
//

CIwGameActor::~CIwGameActor()
{
	OnDestroy();

	// Only delete an unmanaged timeline
	if (Timeline != NULL)
	{
		if (!Timeline->isManaged())
		{
			SAFE_DELETE(Timeline)
		}
	}

	// Delete visual if one was attached
	if (Visual != NULL)
		Visual->getParent()->removeSprite(Visual);

	// Clean up Box2D body object
#if defined(IW_GAME_BOX2D)
	SAFE_DELETE(Box2dBody)
#endif // IW_GAME_BOX2D_H_

	// Delete events manager
	SAFE_DELETE(EventsManager)

	// Delete modifiers
	SAFE_DELETE(Modifiers)
}

void CIwGameActor::setCollisionRect(CIwRect& rect)
{
	CollisionRect = rect;
	int dx = rect.w >> 1;
	int dy = rect.h >> 1;
	CollisionSize = dx * dx + dy * dy;
}

void CIwGameActor::setCollisionSize(int size)
{
	CIwRect rc(-size >> 1, -size >> 1, size, size);
	setCollisionRect(rc);
}

void CIwGameActor::setPosition(float x, float y)
{
	Position.x = x; Position.y = y;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		CIwGameBox2dWorld* world = Scene->getBox2dWorld();
		if (world != NULL)
			body->SetTransform(b2Vec2(world->PixelToWorldX(x), world->PixelToWorldY(y)), body->GetAngle());
	}
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setAngle(float angle)
{
	Angle = angle;
#if defined(IW_GAME_BOX2D)
	Angle = -Angle;
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		body->SetTransform(body->GetPosition(), (Angle * 3.1415927f) / 180.0f);	// Degrees to radians
	}
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setVelocity(float x, float y)
{
	Velocity.x = x;  Velocity.y = y;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		body->SetLinearVelocity(b2Vec2(x, y));
	}
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setVelocityDamping(float x, float y)
{
	VelocityDamping.x = x;  VelocityDamping.y = y;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetLinearDamping(x);
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setVelocityDamping(float damping)
{
	setVelocityDamping(damping, damping);
}

void CIwGameActor::setAngularVelocity(float velocity)
{
	AngularVelocity = velocity;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetAngularVelocity(AngularVelocity);
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setAngularVelocityDamping(float damping)
{
	AngularVelocityDamping = damping;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetAngularVelocity(AngularVelocityDamping);
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setActive(bool active)
{
	if (IsActive == active)
		return;
	IsActive = active;
#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
		body->SetActive(active);
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::setCollidable(bool collidable)
{
	IsCollidable = collidable;
#if defined(IW_GAME_BOX2D)
	if (Box2dBody != NULL)
	{
		if (!collidable)
			Box2dBody->setCollisionFlags(0, 0, 0);
		else
		{
			int flags = Box2dBody->getCollisionFlags();
			Box2dBody->setCollisionFlags(flags >> 16, (flags >> 8) & 0xff, flags & 0xff);
		}
	}
#endif	// IW_GAME_BOX2D
}

void CIwGameActor::Init()
{
	OnCreate();
}

void CIwGameActor::Reset()
{
	IsActive = true;
	IsVisible = true;
	IsCollidable = false;
	IsTappable = true;
	IsDraggable = false;
	IsDragging = false;
	WrapPosition = false;
	Type = 0;
	Position.x = 0;
	Position.y = 0;
	Depth = 1.0f;
	Origin.x = 0;
	Origin.y = 0;
	OriginalPosition.x = 0;
	OriginalPosition.y = 0;
	ScaleX = 1.0f;
	ScaleY = 1.0f;
	OriginalAngle = 0;
	Angle = 0;
	Velocity.x = 0;
	Velocity.y = 0;
	AngularVelocity = 0;
	VelocityDamping.x = 1.0f;
	VelocityDamping.y = 1.0f;
	AngularVelocityDamping = 1.0f;
	CollisionSize = 0;
	CollisionRect.x = 0;
	CollisionRect.y = 0;
	CollisionRect.w = 0;
	CollisionRect.h = 0;
	Colour.r = 0xff;
	Colour.g = 0xff;
	Colour.b = 0xff;
	Colour.a = 0xff;
	Layer = 0;
}

bool CIwGameActor::Update(float dt)
{
	if (!IsActive)
		return true;

	// Update any data bindings
	if (Bindings != NULL)
		UpdateBindings();

	// Update modifiers
	if (Modifiers != NULL)
		Modifiers->Execute(this, dt);

	// Update dragging
	if (IsDraggable)
	{
		if (IsDragging)
		{
			CIwVec2 raw_touch_pos  = IW_GAME_INPUT->getTouchedPos();
			CIwFVec2 cam_pos = Scene->getCamera()->getPosition();
			CIwMat2D& cam_transform = Scene->getCamera()->getTransform();
			cam_pos.x = (cam_pos.x * IW_GEOM_ONE) / cam_transform.m[0][0];
			cam_pos.y = (cam_pos.y * IW_GEOM_ONE) / cam_transform.m[1][1];

			CIwFVec2 virtual_touch_pos = Scene->ScreenToVirtual((float)raw_touch_pos.x, (float)raw_touch_pos.y);
			setPosition(cam_pos.x + virtual_touch_pos.x, cam_pos.y + virtual_touch_pos.y);
		}
	}

#if defined(IW_GAME_BOX2D)
	b2Body* body = (Box2dBody == NULL) ? NULL : Box2dBody->getBody();
	if (body != NULL)
	{
		// Update actor position and angle to match the physics engine
		float x = body->GetPosition().x;
		float y = body->GetPosition().y;
		CIwGameBox2dWorld* world = Scene->getBox2dWorld();
		if (world != NULL)
		{
			Position.x = world->WorldToPixelX(x);
			Position.y = world->WorldToPixelY(y);
		}
		Angle = (body->GetAngle() * 180.0f) / 3.1415927f;	// Radians to degrees
	}
	else
#endif	// IW_GAME_BOX2D
	{
		// Update velocity
		PreviousPosition = Position;
		Position += Velocity * dt;
		Velocity.x *= VelocityDamping.x;
		Velocity.y *= VelocityDamping.y;
		PreviousAngle = Angle;
		Angle += AngularVelocity * dt;
		AngularVelocity *= AngularVelocityDamping;
	}

	// Wrap actor position with scene extents (if enabled)
	CIwRect extents = Scene->getExtents();
	if (WrapPosition)
	{
#if defined(IW_GAME_BOX2D)
		bool wrapped = false;
		if (Position.x < extents.x)
		{
			Position.x += extents.w;
			wrapped = true;
		}
		else
		if (Position.x >= (extents.x + extents.w))
		{
			Position.x -= extents.w;
			wrapped = true;
		}
		if (Position.y < extents.y)
		{
			Position.y += extents.h;
			wrapped = true;
		}
		else
		if (Position.y >= (extents.y + extents.h))
		{
			Position.y -= extents.h;
			wrapped = true;
		}
		if (wrapped && body != NULL)
		{
			CIwGameBox2dWorld* world = Scene->getBox2dWorld();
			if (world != NULL)
				body->SetTransform(b2Vec2(world->PixelToWorldX(Position.x), world->PixelToWorldX(Position.y)), body->GetAngle());
		}
#else
		if (Position.x < extents.x)
			Position.x += extents.w;
		else
		if (Position.x >= (extents.x + extents.w))
			Position.x -= extents.w;
		if (Position.y < extents.y)
			Position.y += extents.h;
		else
		if (Position.y >= (extents.y + extents.h))
			Position.y -= extents.h;
#endif	// IW_GAME_BOX2D
	}

	// Update the animation timeline
	if (Timeline != NULL)
 		Timeline->Update(dt, this);

	// If this actor is collidable then add it to the scenes potential collidables list
	if (IsCollidable)
		Scene->addCollideable(this);

	return true;
}

bool CIwGameActor::UpdateVisual()
{
	if (Visual == NULL)
		return false;

	if (!IsVisible || !IsActive)
	{
		Visual->setVisible(false);
		return false;
	}
	else
	{
		// If ww are linked to another actor then check that it is visible
		if (LinkedTo != NULL)
		{
			if (!LinkedTo->isVisible() || !LinkedTo->isActive())
			{
				Visual->setVisible(false);
				return false;
			}
		}
	}

	Visual->setVisible(true);

	if (LinkedTo != NULL)
		Visual->setLinkedTo(LinkedTo->getVisual());

	Visual->setDepth((iwfixed)(Depth * IW_GEOM_ONE));
	Visual->setOrigin((int)(Origin.x * 8.0f), (int)(Origin.y * 8.0f));
	// Transfor actor properties to the visual
	Visual->setPosAngScale(	(int)(Position.x * 8.0f), 
							(int)(Position.y * 8.0f), 
							(iwangle)((Angle * IW_ANGLE_2PI) / 360.0f), (iwfixed)(ScaleX * IW_GEOM_ONE), (iwfixed)(ScaleY * IW_GEOM_ONE));

	// Multiply actor colour by scene colour
	CIwColour c = Colour;
	CIwColour c2 = Scene->getColour();
	c.r = (uint8)(((int)c.r * c2.r) / 255);
	c.g = (uint8)(((int)c.g * c2.g) / 255);
	c.b = (uint8)(((int)c.b * c2.b) / 255);
	c.a = (uint8)(((int)c.a * c2.a) / 255);
	Visual->setColour(c);

	// Set visible layer
	Visual->setLayer(Layer);

	return true;
}

void CIwGameActor::removeVisual()
{
	if (Visual != NULL)
	{
		Scene->getSpriteManager()->removeSprite(Visual);
		Visual = NULL;
	}
}

CIwGameActor* CIwGameActor::HitTest(int x, int y)
{
	if (Visual == NULL || !isTappable() || !IsVisible)
		return NULL;

	if (Visual->HitTest(x, y))
		return this;		// We return a CIwGameActor so we can return a different actor if this actor hosts child actors

	return NULL;
}

CIwVec2 CIwGameActor::TransformPoint(float x, float y)
{
	if (Visual != NULL)
		return Visual->TransformPoint(x, y);

	return CIwVec2(0, 0);
}

bool CIwGameActor::CheckCollision(CIwGameActor* other)
{
	// This is just about as simple as colision checks can go. We check to see if the distance between the centre of both objects is less than the sum of the radii of both
	// Note that we do not use a proper distance between two points check because that involves expensive square root calculations, but this works just the same
	// The only thing to be careful of is if your coordinate system is huge and your numbers have a large range, just ensure that the term for d = dx * dx + dy * dy does not fall 
	// out of integer range
	CIwFVec2 other_pos = other->getPosition();
	int		other_collision_size = other->getCollisionSize();
	int		dx = (int)(Position.x - other_pos.x);
	int		dy = (int)(Position.y - other_pos.y);
	int		d_max = (CollisionSize >> 1) + (other_collision_size >> 1);
	int		d = dx * dx + dy * dy;

	d_max *= d_max;
	if (d <= d_max)
		return true;

	return false;
}

void CIwGameActor::OnTapped(int x, int y)
{
	ProcessEventActions(CIwGameXomlNames::OnTapped_Hash);
}

void CIwGameActor::OnBeginTouch(int x, int y)
{
	IsDragging = true;
	ProcessEventActions(CIwGameXomlNames::OnBeginTouch_Hash);
}

void CIwGameActor::OnEndTouch(int x, int y)
{
	IsDragging = false;
	ProcessEventActions(CIwGameXomlNames::OnEndTouch_Hash);
}

void CIwGameActor::OnCreate()
{
	ProcessEventActions(CIwGameXomlNames::OnCreate_Hash);
}

void CIwGameActor::OnDestroy()
{
	ProcessEventActions(CIwGameXomlNames::OnDestroy_Hash);
}

void CIwGameActor::ProcessEventActions(unsigned int event_name)
{
	// Find the event
	CIwGameEvent* evt = EventsManager->findEvent(event_name);
	if (evt != NULL)
	{
		// Find actions list
		CIwGameActions* actions = (CIwGameActions*)Scene->getResourceManager()->findResource(evt->ActionsName, CIwGameXomlNames::Actions_Hash);
		if (actions != NULL)
		{
			actions->Execute(this);
		}
	}
}

void CIwGameActor::RemoveBody()
{
	setCollidable(false);
	Velocity.x = 0;
	Velocity.y = 0;
	AngularVelocity = 0;
#if defined(IW_GAME_BOX2D)
	if (Box2dBody != NULL)
	{
		SAFE_DELETE(Box2dBody)
	}
#endif // IW_GAME_BOX2D_H_
}

bool CIwGameActor::setProperty(const char* element_name, CIwGameString& data, bool delta)
{
	return setProperty(CIwGameString::CalculateHash(element_name), data, delta);
}

bool CIwGameActor::setProperty(unsigned int element_name, CIwGameString& data, bool delta)
{
	float float_pool[8];

	if (element_name == CIwGameXomlNames::Position_Hash)
	{
		data.GetAsListOfFloat(float_pool);
		if (delta)
			setPosition(Position.x + float_pool[0], Position.y + float_pool[1]);
		else
			setPosition(float_pool[0], float_pool[1]);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Depth_Hash)
	{
		float d = data.GetAsFloat();
		if (delta)
			setDepth(Depth + d);
		else
			setDepth(d);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Origin_Hash)
	{
		data.GetAsListOfFloat(float_pool);
		if (delta)
		{
			Origin.x += float_pool[0];
			Origin.y += float_pool[1];
		}
		else
		{
			Origin.x = float_pool[0];
			Origin.y = float_pool[1];
		}
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Velocity_Hash)
	{
		data.GetAsListOfFloat(float_pool);
		if (delta)
			setVelocity(Velocity.x + float_pool[0], Velocity.y + float_pool[1]);
		else
			setVelocity(float_pool[0], float_pool[1]);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Angle_Hash)
	{
		float d = data.GetAsFloat();
		if (delta)
			setAngle(Angle + d);
		else
			setAngle(d);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::AngularVelocity_Hash)
	{
		float d = data.GetAsFloat();
		if (delta)
			setAngularVelocity(AngularVelocity + d);
		else
			setAngularVelocity(d);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Scale_Hash)
	{
		data.GetAsListOfFloat(float_pool);
		if (delta)
		{
			ScaleX += float_pool[0];
			ScaleY += float_pool[1];
		}
		else
		{
			ScaleX = float_pool[0];
			ScaleY = float_pool[1];
		}
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::ScaleX_Hash)
	{
		float d = data.GetAsFloat();
		if (delta)
			ScaleX += d;
		else
			ScaleX = d;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::ScaleY_Hash)
	{
		float d = data.GetAsFloat();
		if (delta)
			ScaleY += d;
		else
			ScaleY = d;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Colour_Hash || element_name == CIwGameXomlNames::Color_Hash)
	{
		data.GetAsListOfFloat(float_pool);
		if (delta)
		{
			Colour.r += (uint8)float_pool[0];
			Colour.g += (uint8)float_pool[1];
			Colour.b += (uint8)float_pool[2];
			Colour.a += (uint8)float_pool[3];
		}
		else
		{
			Colour.r = (uint8)float_pool[0];
			Colour.g = (uint8)float_pool[1];
			Colour.b = (uint8)float_pool[2];
			Colour.a = (uint8)float_pool[3];
		}
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Layer_Hash)
	{
		if (delta)
			Layer = data.GetAsInt();
		else
			Layer += data.GetAsInt();
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Visible_Hash)
	{
		if (delta)
			IsVisible = !IsVisible;
		else
			IsVisible = data.GetAsBool();
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Collidable_Hash)
	{
		if (delta)
			setCollidable(!IsCollidable);
		else
			setCollidable(data.GetAsBool());
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Draggable_Hash)
	{
		if (delta)
			setDraggable(!IsDraggable);
		else
			setDraggable(data.GetAsBool());
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::HitTest_Hash)
	{
		if (delta)
			IsTappable = !IsTappable;
		else
			IsTappable = data.GetAsBool();
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Timeline_Hash)
	{
		if (!data.IsEmpty())
		{
			CIwGameAnimTimeline* timeline = (CIwGameAnimTimeline*)Scene->getResourceManager()->findResource(data.c_str(), CIwGameXomlNames::Timeline_Hash);
			if (timeline != NULL && timeline != Timeline)
			{
				setTimeline(timeline);
				timeline->setTargetElement(this);
			}
		}
		return true;
	}
#if defined(IW_GAME_BOX2D_H_)
	else
	if (element_name == CIwGameXomlNames::Torque_Hash)
	{
		if (Box2dBody != NULL)
			Box2dBody->ApplyTorque(data.GetAsFloat());
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::AngularImpulse_Hash)
	{
		if (Box2dBody != NULL)
			Box2dBody->ApplyAngularImpulse(data.GetAsFloat());
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Force_Hash)
	{
		if (Box2dBody != NULL)
		{
			data.GetAsListOfFloat(float_pool);
			Box2dBody->ApplyForce(float_pool[0], float_pool[1], Box2dBody->getWorld()->PixelToWorldX((Position.x + float_pool[2])), Box2dBody->getWorld()->PixelToWorldY(Position.y + float_pool[3]));
		}
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::LinearImpulse_Hash)
	{
		if (Box2dBody != NULL)
		{
			data.GetAsListOfFloat(float_pool);
			Box2dBody->ApplyLinearImpulse(float_pool[0], float_pool[1], Box2dBody->getWorld()->PixelToWorldX(Position.x + float_pool[2]), Box2dBody->getWorld()->PixelToWorldY(Position.y + float_pool[3]));
		}
		return true;
	}
#endif // IW_GAME_BOX2D_H_

	return false;
}

bool CIwGameActor::UpdateFromAnimation(CIwGameAnimInstance *animation)
{
	unsigned int element_name = animation->getTargetPropertyHash();
	bool delta = animation->isDelta();

	if (element_name == CIwGameXomlNames::Position_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		if (delta)
			setPosition(Position.x + frame->data.x, Position.y + frame->data.y);
		else
			setPosition(frame->data.x, frame->data.y);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Depth_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		if (delta)
			setDepth(Depth + frame->data);
		else
			setDepth(frame->data);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Origin_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		if (delta)
		{
			Origin.x += frame->data.x;
			Origin.y += frame->data.y;
		}
		else
		{
			Origin.x = frame->data.x;
			Origin.y = frame->data.y;
		}
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Velocity_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		if (delta)
			setVelocity(Velocity.x + frame->data.x, Velocity.y + frame->data.y);
		else
			setVelocity(frame->data.x, frame->data.y);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Angle_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		if (delta)
			setAngle(Angle + frame->data);
		else
			setAngle(frame->data);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::AngularVelocity_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		if (delta)
			setAngularVelocity(AngularVelocity + frame->data);
		else
			setAngularVelocity(frame->data);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Scale_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		if (delta)
		{
			ScaleX += frame->data.x;
			ScaleY += frame->data.y;
		}
		else
		{
			ScaleX = frame->data.x;
			ScaleY = frame->data.y;
		}
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::ScaleX_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		if (delta)
			ScaleX += frame->data;
		else
			ScaleX = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::ScaleY_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		if (delta)
			ScaleY += frame->data;
		else
			ScaleY = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Colour_Hash || element_name == CIwGameXomlNames::Color_Hash)
	{
		CIwGameAnimFrameVec4* frame = (CIwGameAnimFrameVec4*)animation->getCurrentData();
		if (delta)
		{
			Colour.r += (uint8)frame->data.x;
			Colour.g += (uint8)frame->data.y;
			Colour.b += (uint8)frame->data.z;
			Colour.a += (uint8)frame->data.w;
		}
		else
		{
			Colour.r = (uint8)frame->data.x;
			Colour.g = (uint8)frame->data.y;
			Colour.b = (uint8)frame->data.z;
			Colour.a = (uint8)frame->data.w;
		}
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Layer_Hash)
	{
		CIwGameAnimFrameFloat* frame = (CIwGameAnimFrameFloat*)animation->getCurrentData();
		Layer = (int)frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Visible_Hash)
	{
		CIwGameAnimFrameBool* frame = (CIwGameAnimFrameBool*)animation->getCurrentData();
		IsVisible = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::HitTest_Hash)
	{
		CIwGameAnimFrameBool* frame = (CIwGameAnimFrameBool*)animation->getCurrentData();
		IsTappable = frame->data;
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Timeline_Hash)
	{
		CIwGameAnimFrameString* frame = (CIwGameAnimFrameString*)animation->getCurrentData();
		if (!frame->data.IsEmpty())
		{
			CIwGameAnimTimeline* timeline = (CIwGameAnimTimeline*)Scene->getResourceManager()->findResource(frame->data.c_str(), CIwGameXomlNames::Timeline_Hash);
			if (timeline != NULL && timeline != Timeline)
			{
				setTimeline(timeline);
				timeline->setTargetElement(this);
			}
		}
		return true;
	}

	return false;
}

void CIwGameActor::UpdateBindings()
{
	for (CIwGameXomlBindings::_Iterator it = Bindings->begin(); it != Bindings->end(); ++it)
	{
		unsigned int element_name = (*it)->Property;
		CIwGameXomlVariable* var = (*it)->Variable;
		if (var != NULL)
		{
			if (element_name == CIwGameXomlNames::Position_Hash)
			{
				if (var->Type == VT_Vec2)
				{
					CIwFVec2* data = &((CIwGameXomlVariableVec2*)var)->NativeValue;
					setPosition(data->x, data->y);
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected vec2 for Position");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Origin_Hash)
			{
				if (var->Type == VT_Vec2)
				{
					CIwFVec2* data = &((CIwGameXomlVariableVec2*)var)->NativeValue;
					setOrigin(data->x, data->y);
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected vec2 for Origin");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Angle_Hash)
			{
				if (var->Type == VT_Float)
					setAngle(((CIwGameXomlVariableFloat*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected float for Angle");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Scale_Hash)
			{
				if (var->Type == VT_Float)
					setScale(((CIwGameXomlVariableFloat*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected float for Scale");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::ScaleX_Hash)
			{
				if (var->Type == VT_Float)
					setScaleX(((CIwGameXomlVariableFloat*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected float for ScaleX");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::ScaleY_Hash)
			{
				if (var->Type == VT_Float)
					setScaleY(((CIwGameXomlVariableFloat*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected float for ScaleY");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Colour_Hash || element_name == CIwGameXomlNames::Color_Hash)
			{
				if (var->Type == VT_Vec4)
				{
					CIwFVec4* data = &((CIwGameXomlVariableVec4*)var)->NativeValue;
					Colour.r = (uint8)data->x;
					Colour.g = (uint8)data->y;
					Colour.b = (uint8)data->z;
					Colour.a = (uint8)data->w;
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected vec4 for Colour");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Velocity_Hash)
			{
				if (var->Type == VT_Vec2)
				{
					CIwFVec2* data = &((CIwGameXomlVariableVec2*)var)->NativeValue;
					setVelocity(data->x, data->y);
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected vec2 for Velocity");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::AngularVelocity_Hash)
			{
				if (var->Type == VT_Float)
					setAngularVelocity(((CIwGameXomlVariableFloat*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected float for AngularVelocity");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Timeline_Hash)
			{
				if (var->Type == VT_String)
				{
					if (!var->getValue().IsEmpty())
					{
						CIwGameAnimTimeline* timeline = (CIwGameAnimTimeline*)Scene->getResourceManager()->findResource(var->getValue().c_str(), CIwGameXomlNames::Timeline_Hash);
						if (timeline != NULL && timeline != Timeline)
						{
							setTimeline(timeline);
							timeline->setTargetElement(this);
						}
					}
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected string for timeline");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Binding_Hash)
			{
				if (var->Type == VT_String)
				{
					if (!var->getValue().IsEmpty())
					{
						CIwGameXomlBindings* bindings = (CIwGameXomlBindings*)Scene->getResourceManager()->findResource(var->getValue().c_str(), CIwGameXomlNames::Bindings_Hash);
						setBindings(bindings);
						return;	// Bindings have changed so do not update any further bindnigs
					}
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected string for Bindings");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Depth_Hash)
			{
				if (var->Type == VT_Float)
					Depth = ((CIwGameXomlVariableFloat*)var)->NativeValue;
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected float for Depth");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Type_Hash)
			{
				if (var->Type == VT_Int)
					Type = ((CIwGameXomlVariableInt*)var)->NativeValue;
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected int for Type");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Active_Hash)
			{
				if (var->Type == VT_Bool)
					IsActive = ((CIwGameXomlVariableBool*)var)->NativeValue;
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected bool for Active");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Visible_Hash)
			{
				if (var->Type == VT_Bool)
					IsVisible = ((CIwGameXomlVariableBool*)var)->NativeValue;
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected bool for Visible");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Collidable_Hash)
			{
				if (var->Type == VT_Bool)
					setCollidable(((CIwGameXomlVariableBool*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected bool for Collidable");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Draggable_Hash)
			{
				if (var->Type == VT_Bool)
					setDraggable(((CIwGameXomlVariableBool*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected bool for Draggable");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::HitTest_Hash)
			{
				if (var->Type == VT_Bool)
					setTappable(((CIwGameXomlVariableBool*)var)->NativeValue);
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected bool for HitTest");
#endif // _DEBUG
			}
		}
	}
}


bool CIwGameActor::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	if (parent->getClassTypeHash() != CIwGameXomlNames::Scene_Hash && parent->getClassTypeHash() != CIwGameXomlNames::Actor_Hash)
	{
		CIwGameError::LogError("Error: XOML - Actor needs to be declared inside a scene or an actor");
		return false;
	}

	// Add the actor to the scene
	if (parent->getClassTypeHash() == CIwGameXomlNames::Actor_Hash)
	{
		Scene = ((CIwGameActor*)parent)->getScene();
		LinkedTo = ((CIwGameActor*)parent);	// Because we were defined inside another actor tag
	}
	else
		Scene = (CIwGameScene*)parent;

	Scene->addActor(this);

	// Check and apply style
	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();
		if (name_hash == CIwGameXomlNames::Style_Hash)
		{
			if (!(*it)->GetValue().IsEmpty())
			{
				// Find the style
				CIwGameStyle* style = (CIwGameStyle*)Scene->getResourceManager()->findResource((*it)->GetValue().c_str(), CIwGameXomlNames::Style_Hash);
				if (style == NULL)
				{
					CIwGameError::LogError("Error: XOML - Could not find named style in the scene or in the global resources - ", (*it)->GetValue().c_str());
					return false;
				}
				// Apply the style
				CIwGameXoml::ApplyStyle(style, node);
			}
		}
	}

	// Process Actor attributes
	CIwGameString	timeline_name, box2dmaterial_name, shape_name;
	CIwGameString	bindings_name;
	CIwFVec2		com(0, 0);
	CIwFVec3		collision_flags(0, 0, 0);
	bool			sensor = false;
	bool			vd_set = false;
	bool			avd_set = false;
	bool			is_active = true;
	bool			depth_set = false;
	float			ang_vel = 0;
	CIwFVec2		vel  = CIwFVec2(0.0f, 0.0f);

	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Name_Hash)
		{
			setName((*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::Type_Hash)
			setType((*it)->GetValueAsInt());
		else
		if (name_hash == CIwGameXomlNames::Position_Hash)
		{
			if (!(*it)->GetValueAsPoint(Position))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::Position");
			setOriginalPosition(Position.x, Position.y);
		}
		else
		if (name_hash == CIwGameXomlNames::Origin_Hash)
		{
			if (!(*it)->GetValueAsPoint(Origin))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::Origin");
		}
		else
		if (name_hash == CIwGameXomlNames::Velocity_Hash)
		{
			if (!(*it)->GetValueAsPoint(vel))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::Velocity");
		}
		else
		if (name_hash == CIwGameXomlNames::VelocityDamping_Hash)
		{
			if (!(*it)->GetValueAsPoint(VelocityDamping))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::VelocityDamping");
			else
				vd_set = true;
		}
		else
		if (name_hash == CIwGameXomlNames::Angle_Hash)
		{
			Angle = (*it)->GetValueAsFloat();
			OriginalAngle = Angle;
		}
		else
		if (name_hash == CIwGameXomlNames::AngularVelocity_Hash)
			ang_vel = (*it)->GetValueAsFloat();
		else
		if (name_hash == CIwGameXomlNames::AngularVelocityDamping_Hash)
		{
			AngularVelocityDamping = (*it)->GetValueAsFloat();
			avd_set = true;
		}
		else
		if (name_hash == CIwGameXomlNames::Scale_Hash)
			ScaleX = ScaleY = (*it)->GetValueAsFloat();
		else
		if (name_hash == CIwGameXomlNames::ScaleX_Hash)
			ScaleX = (*it)->GetValueAsFloat();
		else
		if (name_hash == CIwGameXomlNames::ScaleY_Hash)
			ScaleY = (*it)->GetValueAsFloat();
		else
		if (name_hash == CIwGameXomlNames::Colour_Hash || name_hash == CIwGameXomlNames::Color_Hash)
		{
			if (!(*it)->GetValueAsColour(Colour))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::Colour");
		}
		else
		if (name_hash == CIwGameXomlNames::Layer_Hash)
			Layer = (*it)->GetValueAsInt();
		else
		if (name_hash == CIwGameXomlNames::Active_Hash)
			is_active = (*it)->GetValueAsBool();
		else
		if (name_hash == CIwGameXomlNames::Visible_Hash)
			IsVisible = (*it)->GetValueAsBool();
		else
		if (name_hash == CIwGameXomlNames::Collidable_Hash)
			IsCollidable = (*it)->GetValueAsBool();
		else
		if (name_hash == CIwGameXomlNames::Draggable_Hash)
			IsDraggable = (*it)->GetValueAsBool();
		else
		if (name_hash == CIwGameXomlNames::HitTest_Hash)
			IsTappable = (*it)->GetValueAsBool();
		else
		if (name_hash == CIwGameXomlNames::CollisionSize_Hash)
			setCollisionSize((*it)->GetValueAsInt());
		else
		if (name_hash == CIwGameXomlNames::CollisionRect_Hash)
		{
			CIwRect rect;
			if (!(*it)->GetValueAsRect(rect))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::CollisionRect");
			setCollisionRect(rect);
		}
		else
		if (name_hash == CIwGameXomlNames::WrapPosition_Hash)
			WrapPosition = (*it)->GetValueAsBool();
		else
		if (name_hash == CIwGameXomlNames::Timeline_Hash)
			timeline_name = (*it)->GetValue();
		else
		if (name_hash == CIwGameXomlNames::Bindings_Hash)
			bindings_name = (*it)->GetValue();
		else
		if (name_hash == CIwGameXomlNames::Depth_Hash)
		{
			Depth = (*it)->GetValueAsFloat();
			depth_set = true;
		}
		else
		if (name_hash == CIwGameXomlNames::OnTapped_Hash)
			EventsManager->addEvent("OnTapped", (*it)->GetValue().c_str());
		else
		if (name_hash == CIwGameXomlNames::OnBeginTouch_Hash)
			EventsManager->addEvent("OnBeginTouch", (*it)->GetValue().c_str());
		else
		if (name_hash == CIwGameXomlNames::OnEndTouch_Hash)
			EventsManager->addEvent("OnEndTouch", (*it)->GetValue().c_str());
		else
		if (name_hash == CIwGameXomlNames::OnCreate_Hash)
		{
			EventsManager->addEvent("OnCreate", (*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::OnDestroy_Hash)
		{
			EventsManager->addEvent("OnDestroy", (*it)->GetValue().c_str());
		}
		else
		if (name_hash == CIwGameXomlNames::LinkedTo_Hash)
		{
			CIwGameString linked;
			linked = (*it)->GetValue();

			CIwGameActor* actor = Scene->findActor(linked.getHash());
			if (actor != NULL)
				setLinkedTo(actor);
			else
				CIwGameError::LogError("Warning: XOML - Linked actor not found - ", linked.c_str());
		}
#if defined(IW_GAME_BOX2D)
		else
		if (name_hash == CIwGameXomlNames::Box2dMaterial_Hash)
			box2dmaterial_name = (*it)->GetValue();
		else
		if (name_hash == CIwGameXomlNames::Shape_Hash)
			shape_name = (*it)->GetValue();
		else
		if (name_hash == CIwGameXomlNames::COM_Hash)
		{
			if (!(*it)->GetValueAsPoint(com))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::COM");
		}
		else
		if (name_hash == CIwGameXomlNames::Sensor_Hash)
			sensor = (*it)->GetValueAsBool();
		else
		if (name_hash == CIwGameXomlNames::CollisionFlags_Hash)
		{
			if (!(*it)->GetValueAsPoint3(collision_flags))
				CIwGameError::LogError("Warning: XOML - Invalid value for Actor::CollisionFlags");
		}
#endif // IW_GAME_BOX2D
	}

	// If an actor is linked to another and no depth value has been set then it should be set to 0 as depth values for linked actors are accumulative
	if (LinkedTo != NULL && !depth_set)
		Depth = 0;

	// Search the scenes resources for the specified bindings manager
	if (!bindings_name.IsEmpty())
	{
		CIwGameXomlBindings* bindings = (CIwGameXomlBindings*)Scene->getResourceManager()->findResource(bindings_name.c_str(), CIwGameXomlNames::Bindings_Hash);
		if (bindings == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named bindings in the scene or in the global resources - ", bindings_name.c_str());
		}
		setBindings(bindings);
	}

	// Search the scenes resources for the specified timeline
	if (!timeline_name.IsEmpty())
	{
		CIwGameAnimTimeline* timeline = (CIwGameAnimTimeline*)Scene->getResourceManager()->findResource(timeline_name.c_str(), CIwGameXomlNames::Timeline_Hash);
		if (timeline == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named timeline in the scene or in the global resources - ", timeline_name.c_str());
		}
		setTimeline(timeline);
		timeline->setTargetElement(this);
	}

#if defined(IW_GAME_BOX2D)
	// Check for shape and set if present
	CIwGameShape* shape = NULL;
	if (!shape_name.IsEmpty())
	{
		shape = (CIwGameShape*)Scene->getResourceManager()->findResource(shape_name.c_str(), CIwGameXomlNames::Shape_Hash);
		if (shape == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named shape in the scene or in the global resources - ", shape_name.c_str());
		}
	}

	// Check for box2d material and set if present
	if (!box2dmaterial_name.IsEmpty())
	{
		CIwGameBox2dMaterial* material = (CIwGameBox2dMaterial*)Scene->getResourceManager()->findResource(box2dmaterial_name.c_str(), CIwGameXomlNames::Box2dMaterial_Hash);
		if (material == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named material in the scene or in the global resources - ", box2dmaterial_name.c_str());
		}

		if (shape == NULL)
		{
			CIwGameError::LogError("Warning: XOML - Cannot attach Box2d physics to an actor that has no shape");
		}
		else
		{
			Angle = -Angle;
			Box2dBody = new CIwGameBox2dBody();
			if (shape != NULL)
				Box2dBody->setBodyShape(shape);
			Box2dBody->InitBody(Scene->getBox2dWorld(), shape, material, &Position, Angle, com.x, com.y);
			if (!vd_set)
			{
				VelocityDamping.x = 0;
				VelocityDamping.y = 0;
			}
			if (!avd_set)
				AngularVelocityDamping = 0;
			setVelocityDamping(VelocityDamping.x, VelocityDamping.y);
			setAngularVelocityDamping(AngularVelocityDamping);
			Box2dBody->setAsSensor(sensor);
			Box2dBody->setCollisionFlags((int)collision_flags.x, (int)collision_flags.y, (int)collision_flags.z);
			Box2dBody->setUserData(this);
		}
	}
	setVelocity(vel.x, vel.y);
	setAngularVelocity(ang_vel);

#endif // IW_GAME_BOX2D
	setActive(is_active);

	if (load_children)
	{
		// Process inner tags
		if (!IW_GAME_XOML->Process(this, node))
			return false;
	}

	return true;
}

//
//
//
//
//
// CIwGameActorImage implementation
//
//
//
//
//
bool CIwGameActorImage::InitWithImage(CIwGameImage* image, int width, int height)
{
	// Create sprite
	if (image != NULL)
	{
		CIwGameBitmapSprite* sprite = new CIwGameBitmapSprite();
		if (sprite == NULL)
			return false;
			
		// Set sprite image
		sprite->setImage(image);
		sprite->setSrcDest(0, 0, width, height);

		// Set sprite as visual
		Visual = sprite;

		// Add sprite to the sprite manager so it can be managed and drawn
		Scene->getSpriteManager()->addSprite(sprite);
	}

	return true;
}

bool CIwGameActorImage::Init(CIwGameImage* image, int width, int height)
{
	CIwGameActor::Init();

	// Create sprite
	if (!InitWithImage(image, width, height))
		return false;

	return true;
}

bool CIwGameActorImage::Init(CIwGameImage* image, CIwRect* src_rect)
{
	CIwGameActor::Init();

	// Create sprite
	if (!InitWithImage(image, src_rect->w, src_rect->h))
		return false;
	((CIwGameBitmapSprite*)Visual)->setSrcRect(src_rect);

	return true;
}

bool CIwGameActorImage::Init(CIwGameImage* image, int x, int y, int w, int h)
{
	CIwGameActor::Init();

	// Create sprite
	if (!InitWithImage(image, w, h))
		return false;
	((CIwGameBitmapSprite*)Visual)->setSrcRect(x, y, w, h);

	return true;
}


void CIwGameActorImage::SetSrcDest(int x, int y, int w, int h)
{
	if (Visual != NULL)
	{
		((CIwGameBitmapSprite*)Visual)->setSrcDest(x, y, w, h);
	}
}

void CIwGameActorImage::SetSrcRect(int x, int y, int w, int h)
{
	if (Visual != NULL)
	{
		((CIwGameBitmapSprite*)Visual)->setSrcRect(x, y, w, h);
	}
}

void CIwGameActorImage::SetDestSize(int w, int h)
{
	if (Visual != NULL)
	{
		((CIwGameBitmapSprite*)Visual)->setDestSize(w, h);
	}
}

void CIwGameActorImage::SetImage(CIwGameImage* image)
{
	if (image != NULL)
	{
		if (Visual == NULL)
		{
			CIwGameBitmapSprite* sprite = new CIwGameBitmapSprite();

			// Set sprite image
			sprite->setImage(image);

			// Set sprite as visual
			Visual = sprite;

			// Add sprite to the sprite manager so it can be managed and drawn
			Scene->getSpriteManager()->addSprite(sprite);
		}
		else
		{
			CIwGameBitmapSprite* sprite = (CIwGameBitmapSprite*)Visual;

			// Set sprite image
			sprite->setImage(image);
		}
	}
}

void CIwGameActorImage::SetFromBrush(CIwGameBrushImage* brush)
{
	SetImage(brush->getImage());
	CIwGameBitmapSprite* sprite = (CIwGameBitmapSprite*)Visual;
	sprite->setSrcRect(&brush->getSrcRect());
}


bool CIwGameActorImage::UpdateVisual()
{
	if (CIwGameActor::UpdateVisual())
	{
	}

	return false;
}

bool CIwGameActorImage::setProperty(unsigned int element_name, CIwGameString& data, bool delta)
{
	if (CIwGameActor::setProperty(element_name, data, delta))
		return true;

	int int_pool[8];

	if (element_name == CIwGameXomlNames::SrcRect_Hash)
	{
		data.GetAsListOfInt(int_pool);
		if (delta)
		{
			CIwRect rect = ((CIwGameBitmapSprite*)Visual)->getSrcRect();
			rect.x += int_pool[0];
			rect.y += int_pool[1];
			rect.w += int_pool[2];
			rect.h += int_pool[3];
			((CIwGameBitmapSprite*)Visual)->setSrcRect(&rect);
		}
		else
			((CIwGameBitmapSprite*)Visual)->setSrcRect(int_pool[0], int_pool[1], int_pool[2], int_pool[3]);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Size_Hash)
	{
		data.GetAsListOfInt(int_pool);
		if (delta)
		{
			int width = Visual->getDestWidth();
			int height = Visual->getDestHeight();
			width += int_pool[0];
			height += int_pool[1];
			Visual->setDestSize(width, height);
		}
		else
			Visual->setDestSize(int_pool[0], int_pool[1]);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Image_Hash)
	{
		CIwGameImage* image = (CIwGameImage*)Scene->getResourceManager()->findResource(data.getHash(), CIwGameXomlNames::Image_Hash);
		if (image != NULL)
			SetImage(image);
		else
			CIwGameError::LogError("Warning: setProperty image not found in resources - ", data.c_str());
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Brush_Hash)
	{
		CIwGameBrushImage* brush = (CIwGameBrushImage*)Scene->getResourceManager()->findResource(data.getHash(), CIwGameXomlNames::Brush_Hash);
		if (brush != NULL)
			SetFromBrush(brush);
		else
			CIwGameError::LogError("Warning: setProperty brush not found in resources - ", data.c_str());
		return true;
	}

	return false;
}

bool CIwGameActorImage::UpdateFromAnimation(CIwGameAnimInstance *animation)
{
	if (CIwGameActor::UpdateFromAnimation(animation))
		return true;

	unsigned int element_name = animation->getTargetPropertyHash();

	if (element_name == CIwGameXomlNames::SrcRect_Hash)
	{
		CIwGameAnimFrameRect* frame = (CIwGameAnimFrameRect*)animation->getCurrentData();
		((CIwGameBitmapSprite*)Visual)->setSrcRect(&frame->data);
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Size_Hash)
	{
		CIwGameAnimFrameVec2* frame = (CIwGameAnimFrameVec2*)animation->getCurrentData();
		((CIwGameBitmapSprite*)Visual)->setDestSize((int)frame->data.x, (int)frame->data.y);
		return true;
	}

	return false;
}

void CIwGameActorImage::UpdateBindings()
{
	CIwGameActor::UpdateBindings();

	for (CIwGameXomlBindings::_Iterator it = Bindings->begin(); it != Bindings->end(); ++it)
	{
		unsigned int element_name = (*it)->Property;
		CIwGameXomlVariable* var = (*it)->Variable;
		if (var != NULL)
		{
			if (element_name == CIwGameXomlNames::Size_Hash)
			{
				if (var->Type == VT_Vec2)
				{
					CIwFVec2* data = &((CIwGameXomlVariableVec2*)var)->NativeValue;
					SetDestSize((int)data->x, (int)data->y);
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected vec2 for Size");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::SrcRect_Hash)
			{
				if (var->Type == VT_Vec4)
				{
					CIwFVec4* data = &((CIwGameXomlVariableVec4*)var)->NativeValue;
					SetSrcRect((int)data->x, (int)data->y, (int)data->z, (int)data->w);
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected vec4 for SrcRect");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::Image_Hash)
			{
				if (var->Type == VT_String)
				{
					CIwGameImage* image = (CIwGameImage*)Scene->getResourceManager()->findResource(var->getValue().c_str(), CIwGameXomlNames::Image_Hash);
					CIwGameBitmapSprite* sprite = (CIwGameBitmapSprite*)Visual;
					if (sprite != NULL && sprite->getImage() != image)
						sprite->setImage(image);
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected string for Image");
#endif // _DEBUG
			}
		}
	}
}

bool CIwGameActorImage::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	if (parent->getClassTypeHash() != CIwGameXomlNames::Scene_Hash && parent->getClassTypeHash() != CIwGameXomlNames::Actor_Hash)
	{
		CIwGameError::LogError("Error: XOML - Actor needs to be declared inside a scene or an actor");
		return false;
	}

	// Load main actor attributes
	if (!CIwGameActor::LoadFromXoml(parent, false, node))
		return false;

	// Process ActorImage specific attributes
	CIwGameString	image_name;
	CIwGameString	brush_name;
	CIwFVec2		size;
	CIwRect			src_rect(0, 0, 0, 0);
	int				image_transform = IW_2D_IMAGE_TRANSFORM_NONE;

	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Image_Hash)
		{
			image_name = (*it)->GetValue();
		}
		else
		if (name_hash == CIwGameXomlNames::Size_Hash)
		{
			if (!(*it)->GetValueAsPoint(size))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for ActorImage::Size");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::SrcRect_Hash)
		{
			if (!(*it)->GetValueAsRect(src_rect))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for ActorImage::SrcRect");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::FlipX_Hash)
		{
			if ((*it)->GetValueAsBool())
				image_transform |= IW2D_FLIP_X_F;
		}
		else
		if (name_hash == CIwGameXomlNames::FlipY_Hash)
		{
			if ((*it)->GetValueAsBool())
				image_transform |= IW2D_FLIP_Y_F;
		}
		else
		if (name_hash == CIwGameXomlNames::Brush_Hash)
		{
			brush_name = (*it)->GetValue();
		}
	}

	CIwGameBrushImage* brush = NULL;
	CIwGameImage* image = NULL;
	if (!brush_name.IsEmpty())
	{
		brush = (CIwGameBrushImage*)Scene->getResourceManager()->findResource(brush_name.getHash(), CIwGameXomlNames::Brush_Hash);
		if (brush != NULL)
		{
			image = brush->getImage();
			src_rect = brush->getSrcRect();
		}
		else
			CIwGameError::LogError("Warning: Could not find named brush in the scene or in the global resources - ", brush_name.c_str());
	}
	else
	{
		// Search the scenes resources for the specified image
		image = (CIwGameImage*)Scene->getResourceManager()->findResource(image_name.c_str(), CIwGameXomlNames::Image_Hash);
		if (image == NULL)
		{
			CIwGameError::LogError("Error: XOML - Could not find named image in the scene or in the global resources - ", image_name.c_str());
			return false;
		}
	}

	// Initialise the image actor
	Init(image, (int)size.x, (int)size.y);
	if (src_rect.w != 0)
	{
		CIwGameBitmapSprite* visual = (CIwGameBitmapSprite*)Visual;
		visual->setSrcRect(&src_rect);
		visual->setImageTransform((CIw2DImageTransform)image_transform);
	}

	// Process inner tags
	if (load_children)
	{
		if (!IW_GAME_XOML->Process(this, node))
			return false;
	}

	return true;
}

//
//
//
//
//
// CIwGameActorText implementation
//
//
//
//
//

void CIwGameActorText::setText(const char* text)
{
	CIwGameTextSprite* visual = (CIwGameTextSprite*)Visual;
	if (visual != NULL)
		visual->setText(text);
}

void CIwGameActorText::setRect(CIwRect& rect)
{
	CIwGameTextSprite* visual = (CIwGameTextSprite*)Visual;
	if (visual != NULL)
		visual->setRect(rect);
}

void CIwGameActorText::setFlags(IwGxFontFlags flags)
{
	CIwGameTextSprite* visual = (CIwGameTextSprite*)Visual;
	if (visual != NULL)
		visual->setFlags(flags);
}
void CIwGameActorText::setFont(CIwGameFont* font)
{
	CIwGameTextSprite* visual = (CIwGameTextSprite*)Visual;
	if (visual != NULL)
		visual->setFont(font);
}

void CIwGameActorText::setAlignH(IwGxFontAlignHor align)
{
	CIwGameTextSprite* visual = (CIwGameTextSprite*)Visual;
	if (visual != NULL)
		visual->setAlignH(align);
}

void CIwGameActorText::setAlignV(IwGxFontAlignVer align)
{
	CIwGameTextSprite* visual = (CIwGameTextSprite*)Visual;
	if (visual != NULL)
		visual->setAlignV(align);
}

bool CIwGameActorText::Init(CIwGameFont* font)
{
	CIwGameActor::Init();

	// Create sprite
	if (font != NULL)
	{
		CIwGameTextSprite* sprite = new CIwGameTextSprite();
		if (sprite == NULL)
			return false;

		// Set sprite image
		sprite->setFont(font);

		// Set sprite as visual
		Visual = sprite;

		// Add sprite to the sprite manager so it can be managed and drawn
		Scene->getSpriteManager()->addSprite(sprite);
	}

	return true;
}

bool CIwGameActorText::UpdateVisual()
{
	if (CIwGameActor::UpdateVisual())
	{
	}

	return false;
}

bool CIwGameActorText::setProperty(unsigned int element_name, CIwGameString& data, bool delta)
{
	if (CIwGameActor::setProperty(element_name, data, delta))
		return true;

	if (element_name == CIwGameXomlNames::Text_Hash)
	{
		setText(data.c_str());
		return true;
	}
	else
	if (element_name == CIwGameXomlNames::Font_Hash)
	{
		CIwGameFont* font = (CIwGameFont*)Scene->getResourceManager()->findResource(data.getHash(), CIwGameXomlNames::Font_Hash);
		if (font != NULL)
			setFont(font);
		else
			CIwGameError::LogError("Warning: setProperty font not found in resources - ", data.c_str());
		return true;
	}

	return false;
}

bool CIwGameActorText::UpdateFromAnimation(CIwGameAnimInstance *animation)
{
	if (CIwGameActor::UpdateFromAnimation(animation))
		return true;

	return false;
}

void CIwGameActorText::UpdateBindings()
{
	CIwGameActor::UpdateBindings();

	for (CIwGameXomlBindings::_Iterator it = Bindings->begin(); it != Bindings->end(); ++it)
	{
		unsigned int element_name = (*it)->Property;
		CIwGameXomlVariable* var = (*it)->Variable;
		if (var != NULL)
		{
			if (element_name == CIwGameXomlNames::Text_Hash)
			{
				if (var->Type == VT_String)
				{
					if (Visual != NULL && ((CIwGameTextSprite*)Visual)->getText().getHash() != var->getValue().getHash())
						((CIwGameTextSprite*)Visual)->setText(var->getValue().c_str());
				}
#if defined(_DEBUG)
			else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected string for Text");
#endif // _DEBUG
			}
			else
			if (element_name == CIwGameXomlNames::rect_Hash)
			{
				if (var->Type == VT_Vec4)
				{
					CIwFVec4* data = &((CIwGameXomlVariableVec4*)var)->NativeValue;
					CIwRect rc((int)data->x, (int)data->y, (int)data->z, (int)data->w);
					setRect(rc);
				}
#if defined(_DEBUG)
				else
					CIwGameError::LogError("Warning: Incorrect binding variable type, expected vec4 for SrcRect");
#endif // _DEBUG
			}
		}
	}
}


bool CIwGameActorText::LoadFromXoml(IIwGameXomlResource* parent, bool load_children, CIwGameXmlNode* node)
{
	if (parent->getClassTypeHash() != CIwGameXomlNames::Scene_Hash && parent->getClassTypeHash() != CIwGameXomlNames::Actor_Hash)
	{
		CIwGameError::LogError("Error: XOML - Actor needs to be declared inside a scene or an actor");
		return false;
	}

	// Load main actor attributes
	if (!CIwGameActor::LoadFromXoml(parent, false, node))
		return false;

	// Process Actor Font specific attributes
	CIwGameString	font_name;
	CIwGameString	text;
	CIwRect			rect;
	IwGxFontAlignHor alignh = IW_GX_FONT_ALIGN_CENTRE;
	IwGxFontAlignVer alignv = IW_GX_FONT_ALIGN_MIDDLE;
	int				flags = 0;

	for (CIwGameXmlNode::_AttribIterator it = node->attribs_begin(); it != node->attribs_end(); it++)
	{
		unsigned int name_hash = (*it)->getName().getHash();

		if (name_hash == CIwGameXomlNames::Font_Hash)
		{
			font_name = (*it)->GetValue();
		}
		else
		if (name_hash == CIwGameXomlNames::rect_Hash)
		{
			if (!(*it)->GetValueAsRect(rect))
			{
				CIwGameError::LogError("Warning: XOML - Invalid value for ActorText::Rect");
			}
		}
		else
		if (name_hash == CIwGameXomlNames::Text_Hash)
		{
			text = (*it)->GetValue();
		}
		else
		if (name_hash == CIwGameXomlNames::AlignH_Hash)
		{
			unsigned int align_hash = (*it)->GetValue().getHash();
			if (align_hash == CIwGameXomlNames::left_Hash)
				alignh = IW_GX_FONT_ALIGN_LEFT;
			else
			if (align_hash == CIwGameXomlNames::centre_Hash)
				alignh = IW_GX_FONT_ALIGN_CENTRE;
			else
			if (align_hash == CIwGameXomlNames::right_Hash)
				alignh = IW_GX_FONT_ALIGN_RIGHT;
		}
		else
		if (name_hash == CIwGameXomlNames::AlignV_Hash)
		{
			unsigned int align_hash = (*it)->GetValue().getHash();
			if (align_hash == CIwGameXomlNames::top_Hash)
				alignv = IW_GX_FONT_ALIGN_TOP;
			else
			if (align_hash == CIwGameXomlNames::middle_Hash)
				alignv = IW_GX_FONT_ALIGN_MIDDLE;
			else
			if (align_hash == CIwGameXomlNames::bottom_Hash)
				alignv = IW_GX_FONT_ALIGN_BOTTOM;
		}
		else
		if (name_hash == CIwGameXomlNames::Wrap_Hash)
		{
			if ((*it)->GetValueAsBool())
				flags |= IW_GX_FONT_DEFAULT_F;
			else
				flags |= IW_GX_FONT_NOWRAP_F;
		}
		else
		if (name_hash == CIwGameXomlNames::Italic_Hash)
		{
			if ((*it)->GetValueAsBool())
				flags |= IW_GX_FONT_ITALIC_F;
		}
		else
		if (name_hash == CIwGameXomlNames::Underline_Hash)
		{
			if ((*it)->GetValueAsBool())
				flags |= IW_GX_FONT_UNDERLINE_F;
		}
	}

	// Search the scenes resources for the specified font
	CIwGameFont* font = (CIwGameFont*)Scene->getResourceManager()->findResource(font_name.c_str(), CIwGameXomlNames::Font_Hash);
	if (font == NULL)
	{
		CIwGameError::LogError("Error: XOML - Could not find named font in the scene or in the global resources - ", font_name.c_str());
		return false;
	}

	// Initialise the image actor
	Init(font);
	CIwGameTextSprite* visual = (CIwGameTextSprite*)Visual;
	visual->setRect(rect);
	visual->setText(text.c_str());
	visual->setFlags((IwGxFontFlags)flags);
	Visual->setDestSize(rect.w, rect.h);

	// Set alignment
	setAlignH(alignh);
	setAlignV(alignv);

	// Process inner tags
	if (load_children)
	{
		if (!IW_GAME_XOML->Process(this, node))
			return false;
	}

	return true;
}

//
//
//
//
//  CIwGameXomlAction_Actors implementation
//
//
//
//
void CIwGameXomlAction_Actors::Execute(IIwGameXomlResource* source, CIwGameAction* action)
{
	CIwGame* game = NULL;
	CIwGameScene* scene = NULL;
	CIwGameActor* actor = NULL;

	if (source->getClassTypeHash() == CIwGameXomlNames::Scene_Hash)
	{
		scene = (CIwGameScene*)source;
		game = scene->getParent();
	}
	else
	if (source->getClassTypeHash() == CIwGameXomlNames::Actor_Hash)
	{
		actor = (CIwGameActor*)source;
		scene = actor->getScene();
		game = scene->getParent();
	}

	if (Type == Action_SetProperty || Type == Action_AddProperty)
	{
		if (!action->Parameter4.IsEmpty())	// Check for a specific scene
		{
			CIwGameScene* new_scene = game->findScene(action->Parameter4.getHash());
			if (new_scene != NULL)
				scene = new_scene;
			else
				CIwGameError::LogError("Warning: Set(Add)Property - Scene was not found - ", action->Parameter4.c_str());
		}
		if (!action->Parameter3.IsEmpty())	// Check for a specific actor
		{
			CIwGameActor* new_actor = scene->findActor(action->Parameter3.getHash());
			if (new_actor != NULL)
				actor = new_actor;
			else
				CIwGameError::LogError("Warning: Set(Add)Property - Actor was not found - ", action->Parameter3.c_str());
		}
		actor->setProperty(action->Parameter1.getHash(), action->Parameter2, Type == Action_AddProperty);
		return;
	}

	CIwGameActor* new_actor = action->Parameter1.IsEmpty() ? actor : scene->findActor(action->Parameter1.getHash());
	if (new_actor != NULL)
	{
		switch (Type)
		{
		case Action_HideActor:
			new_actor->setVisible(false);
			break;
		case Action_ShowActor:
			new_actor->setVisible(true);
			break;
		case Action_ActivateActor:
			new_actor->setActive(true);
			break;
		case Action_DeactivateActor:
			new_actor->setActive(false);
			break;
		case Action_KillActor:
			scene->removeActor(new_actor);
			break;
		}
	}
	else
		CIwGameError::LogError("Warning: Actions - Actor was not found - ", action->Parameter1.c_str());
}

